/**
 * Create an ABBR + Update ABBR or ACRONYM
 * Released under Attribution-ShareAlike 4.0 International License.
 * Author: Ignacio Gros (http://gros.es/) for http://exelearning.net/
 * License: http://creativecommons.org/licenses/by-sa/4.0/ 
 */

tinymce.PluginManager.add('abbr', function(editor) {
	function showDialog() {

		// Default values (empty)		
		var abbr = "";
		var meaning = "";

		// Update those values if we're in an ABBR or an ACRONYM
		var node = editor.selection.getNode();
		if (node.nodeName=="ABBR" || node.nodeName=="ACRONYM") {
			abbr = node.innerHTML;
			meaning = node.title;			
		}

		// Open the dialog
		editor.windowManager.open({
			title: _('Acronym or abbreviation'),
			body: [
				{
					type: 'textbox', 
					name: 'abbr', 
					size: 40, 
					label: _('Abbreviation'), 
					value: abbr
				},
				{
					type: 'textbox', 
					name: 'meaning', 
					size: 40, 
					label: _('Meaning'), 
					value: meaning
				}
			],				
			onsubmit: function(e) {  
			
				var abbr = e.data.abbr;
				var meaning = e.data.meaning;
				
				// Both fields are required
				if (abbr=="" || meaning=="") {
					editor.windowManager.alert(_('Please write the abbreviation and its meaning.'));
					return false;
				}

				var node = editor.selection.getNode();
				
				// Update the previous ABBR or ACRONYM
				if (node.nodeName=="ABBR" || node.nodeName=="ACRONYM") {
					
					if (node.nodeName=="ACRONYM" && editor.settings.schema=="html5") {
						
						// It's html5 and ACRONYM is being used, so we ask the user:
						editor.windowManager.confirm(
							_('The <acronym> tag is not supported in HTML5.\n\nUse the <abbr> tag instead?'),
							function(s){
								if (s) {
									
									// Remove the ACRONYM and create an ABBR
									try {
                                        editor.dom.remove(node);
                                        editor.selection.setNode(tinymce.activeEditor.dom.create('abbr', {title: meaning},abbr));
                                        editor.undoManager.add();
                                        editor.windowManager.close();
                                    } catch(e) {
                                        // IE 11 won't close the dialog (see https://github.com/exelearning/iteexe/issues/124)
                                        editor.windowManager.alert(_("Failed to save properties"));
                                    }
									
								} else {
									
									// Just update the ACRONYM
									editor.dom.setAttrib(node, "title", meaning);
									editor.dom.setHTML(node, abbr);
									editor.undoManager.add();
									editor.windowManager.close();
									
								}
							}
						);
						
					} else {
						
						// Update the node (ABBR or ACRONYM)
						editor.dom.setAttrib(node, "title", meaning);
						editor.dom.setHTML(node, abbr);
						editor.undoManager.add();
						editor.windowManager.close();
						
					}
				} else {
					
					// Create a new ABBR
					editor.selection.setNode(tinymce.activeEditor.dom.create('abbr', {title: meaning},abbr));
					editor.undoManager.add();
					editor.windowManager.close();
					
				}
				
				return false;
				
			}
		});
	}

	editor.addMenuItem('abbr', {
		text: _('Acronym or abbreviation'),
		context: 'insert',
		onclick: showDialog
	});
	
});