/**
 * Toolbar Toggler Plugin
 * Released under Attribution-ShareAlike 4.0 International License.
 * Author: Ignacio Gros (http://gros.es/)
 * License: http://creativecommons.org/licenses/by-sa/4.0/ 
 * Based in the wp_adv button (wordpress plugin)
 * Icon by WordPress
 */
tinymce.PluginManager.add('toggletoolbars', function(editor, url) {
	
	tinymceToolbarToggler = {
		
		state : "visibleToolbars",
		
		check : function(skipCheck){
			
			// Toolbars
			// To select all the toolbars but the menu bar: editor.theme.panel.find('.toolbar:not(.menubar)');
			var toolbars = editor.theme.panel.find('.toolbar');
			
			if (!toolbars || toolbars.length<3) return;
			
			var state = tinymceToolbarToggler.state;
			
			if (skipCheck!=true && tinymce.editors.length>1) {
				tinymce.each(toolbars,function(toolbar,i){
					if (i==1) {
						if (toolbar.visible()) state = 'hiddenToolbars';
						else state = 'visibleToolbars';
					}
				});	
			}
			
			var newState = "visibleToolbars";
			if (state=='visibleToolbars') newState = 'hiddenToolbars';
			
			tinymce.each(toolbars,function(toolbar,i){
				if (i>0) {
					if (i!=1) {
						if (state=='visibleToolbars') toolbar.hide();
						else toolbar.show();
					} else {
						if (state=='visibleToolbars') toolbar.show();
						else toolbar.hide();
					}
				}
			});				
			
			tinymceToolbarToggler.state = newState;
			
			// Remember the last position
			tinymceToolbarToggler.cookie.set("tinymceToolbarTogglerPreference", newState, 30);			
			
		},
		cookie : {

			set : function(cname,cvalue,exdays) {
				var d = new Date();
				d.setTime(d.getTime() + (exdays*24*60*60*1000));
				var expires = "expires=" + d.toGMTString();
				document.cookie = cname+"="+cvalue+"; "+expires;
			},

			get : function(cname) {
				var name = cname + "=";
				var ca = document.cookie.split(';');
				for(var i=0; i<ca.length; i++) {
					var c = ca[i];
					while (c.charAt(0)==' ') {
						c = c.substring(1);
					}
					if (c.indexOf(name) == 0) {
						return c.substring(name.length, c.length);
					}
				}
				return "";
			}
			
		}		
		
	} // tinymceToolbarToggler
	
	editor.addButton('toggletoolbars', {
		image: url + '/img/toggletoolbars.png',
		tooltip: _('Toolbar Toggle'),
		onclick: tinymceToolbarToggler.check		
	});

	// Hide the toolbars after loading
	editor.on( 'init', function() {
		
		// Last position
		
		// Default: Hide the toolbar
		
		// Except in multiple-visible mode when there's more than one editor (to review)
		if (tinymce.editors.length>1 && $exeTinyMCE.mode == "multiple-visible") tinymceToolbarToggler.cookie.set("tinymceToolbarTogglerPreference", "visibleToolbars", 30);			
		
		var ck = tinymceToolbarToggler.cookie.get("tinymceToolbarTogglerPreference");		
		if (!ck || ck!="visibleToolbars") {
			
			tinymceToolbarToggler.check(true); // true = skipCheck
			
		} else {
			
			// Hide the toolbar toggler
			var toolbars = editor.theme.panel.find('.toolbar');
			
			if (!toolbars || toolbars.length<3) return;
			
			tinymce.each(toolbars,function(toolbar,i){
				if (i==1) toolbar.hide();
			});
			
		}
		
	});		
	
});